# -*- coding: utf-8 -*-

import click
import time


@click.group()
def zb():
    pass


@zb.command()
@click.option('-p', '--path', help="文件路径")
@click.option('-f', '--f', help="文件名中需要修改的内容")
@click.option('-t', '--t', help="修改之后的内容")
def brf(path, f, t):
    """batch_rename_file | 批量重命名文件"""
    from .tools.file import batch_rename_file
    batch_rename_file(path, f, t)


@zb.command()
def ts():
    """timestamp | 查看当前时间戳"""
    print("当前时间戳：", time.time())


if __name__ == "__main__":
    zb()



# -*- coding: utf-8 -*-

from collections import OrderedDict


class AttrDict(dict):
    """Dict that can get attribute by dot"""

    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self


class OrderedAttrDict(OrderedDict):
    """OrderedDict that can get attribute by dot"""

    def __init__(self, *args, **kwargs):
        super(OrderedAttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self


class ZbList(list):
    """List that can get elements by discrete indices"""

    def __init__(self, *args, **kwargs):
        super(ZbList, self).__init__(*args, **kwargs)

    def discrete_index(self, indices):
        """get elements by discrete indices

        :param indices: list
            discrete indices
        :return: elements
        """
        elements = []
        for i in indices:
            elements.append(self[i])
        return elements



# -*- coding: utf-8 -*-

import functools
import time


# --------------------------------------------------------------------

def elapsed(func):
    """A decorator for calculating time elapsed when execute func"""

    @functools.wraps(func)
    def wrapper(*args, **kw):
        start = time.time()
        print('Running %s() ...' % func.__name__)
        res = func(*args, **kw)
        end = time.time()
        print('Function %s() running elapsed %.2f s' %
              (func.__name__, end - start))
        return res

    return wrapper


# --------------------------------------------------------------------

def create_logger(log_file, name='logger', cmd=True):
    """define a logger for your program

    parameters
    ------------
    log_file     file name of log
    name         name of logger

    example
    ------------
    logger = create_logger('example.log',name='logger',)
    logger.info('This is an example!')
    logger.warning('This is a warn!')

    """
    import logging
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    # set format
    formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s',
                                  datefmt='%Y-%m-%d %H:%M:%S')

    # file handler
    fh = logging.FileHandler(log_file)
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    # cmd handler
    if cmd:
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        ch.setFormatter(formatter)
        logger.addHandler(ch)

    return logger



# -*- coding: utf-8 -*-

__version__ = "0.0.5"
__author__ = "zengbin93"


from zb.utils import elapsed, create_logger
import zb.functions as functions
import zb.nlp as nlp
import zb.tools as tools
from zb.base import OrderedAttrDict, AttrDict




# -*- coding: utf-8 -*-

"""
加载数据集
"""
import pandas as pd


def load_iris():
    # The “Iris” dataset  https://archive.ics.uci.edu/ml/datasets/Iris
    path = 'https://raw.githubusercontent.com/pandas-dev/pandas/master/pandas/tests/data/iris.csv'
    iris = pd.read_csv(path)
    return iris




# -*- coding: utf-8 -*-
"""
————————————————————————————————————————————————————————————————————————————————————
单隐层神经网络的实现

step 1. 构造网络
step 2. 参数初始化
step 3. 迭代计算（循环）更新参数
loop
    前馈计算
    后馈计算
    使用梯度下降法更新参数
until 参数更新很小 or 到达迭代上限
step 4. 将以上内容组合成模型
step 5. 编写预测（prediction）函数
————————————————————————————————————————————————————————————————————————————————————
"""

import numpy as np
from ..functions import sigmoid


def layer_sizes(X, Y):
    """
    Arguments:
    X -- input dataset of shape (input size, number of examples)
    Y -- labels of shape (output size, number of examples)

    Returns:
    n_x -- the size of the input layer
    n_h -- the size of the hidden layer
    n_y -- the size of the output layer
    """
    n_x = X.shape[0]  # size of input layer
    n_h = 4
    n_y = Y.shape[0]  # size of output layer
    return (n_x, n_h, n_y)


def initialize_parameters(n_x, n_h, n_y):
    """
    Argument:
    n_x -- size of the input layer
    n_h -- size of the hidden layer
    n_y -- size of the output layer

    Returns:
    params -- python dictionary containing your parameters:
                    W1 -- weight matrix of shape (n_h, n_x)
                    b1 -- bias vector of shape (n_h, 1)
                    W2 -- weight matrix of shape (n_y, n_h)
                    b2 -- bias vector of shape (n_y, 1)
    """

    np.random.seed(2)

    W1 = np.random.randn(n_h, n_x) * 0.01
    b1 = np.zeros((n_h, 1))
    W2 = np.random.randn(n_y, n_h) * 0.01
    b2 = np.zeros((n_y, 1))

    assert (W1.shape == (n_h, n_x))
    assert (b1.shape == (n_h, 1))
    assert (W2.shape == (n_y, n_h))
    assert (b2.shape == (n_y, 1))

    parameters = {"W1": W1,
                  "b1": b1,
                  "W2": W2,
                  "b2": b2}

    return parameters


def forward_propagation(X, parameters):
    """
    Argument:
    X -- input data of size (n_x, m)
    parameters -- python dictionary containing your parameters (output of initialization function)

    Returns:
    A2 -- The sigmoid output of the second activation
    cache -- a dictionary containing "Z1", "A1", "Z2" and "A2"
    """
    # Retrieve each parameter from the dictionary "parameters"
    W1 = parameters["W1"]
    b1 = parameters["b1"]
    W2 = parameters["W2"]
    b2 = parameters["b2"]

    # Implement Forward Propagation to calculate A2 (probabilities)
    Z1 = np.dot(W1, X) + b1
    A1 = np.tanh(Z1)
    Z2 = np.dot(W2, A1) + b2
    A2 = sigmoid(Z2)

    assert (A2.shape == (1, X.shape[1]))

    cache = {"Z1": Z1,
             "A1": A1,
             "Z2": Z2,
             "A2": A2}

    return A2, cache


def compute_cost(A2, Y):
    """
    Computes the cross-entropy cost given in equation (13)

    Arguments:
    A2 -- The sigmoid output of the second activation, of shape (1, number of examples)
    Y -- "true" labels vector of shape (1, number of examples)
    parameters -- python dictionary containing your parameters W1, b1, W2 and b2

    Returns:
    cost -- cross-entropy cost given equation (13)
    """

    m = Y.shape[1]  # number of example

    # Compute the cross-entropy cost
    logprobs = np.multiply(np.log(A2), Y) + np.multiply(np.log(1 - A2), (1 - Y))
    cost = -np.sum(logprobs) / m

    cost = np.squeeze(cost)  # makes sure cost is the dimension we expect.
    # E.g., turns [[17]] into 17

    assert (isinstance(cost, float))

    return cost


def backward_propagation(parameters, cache, X, Y):
    """
    Implement the backward propagation using the instructions above.

    Arguments:
    parameters -- python dictionary containing our parameters
    cache -- a dictionary containing "Z1", "A1", "Z2" and "A2".
    X -- input data of shape (2, number of examples)
    Y -- "true" labels vector of shape (1, number of examples)

    Returns:
    grads -- python dictionary containing your gradients with respect to different parameters
    """
    m = X.shape[1]

    # First, retrieve W1 and W2 from the dictionary "parameters".
    W1 = parameters["W1"]
    W2 = parameters["W2"]

    # Retrieve also A1 and A2 from dictionary "cache".
    A1 = cache["A1"]
    A2 = cache["A2"]

    # Backward propagation: calculate dW1, db1, dW2, db2.
    dZ2 = A2 - Y
    dW2 = 1.0 / m * np.dot(dZ2, A1.T)
    db2 = 1.0 / m * np.sum(dZ2, axis=1, keepdims=True)
    dZ1 = W2.T * dZ2 * (1 - np.power(A1, 2))
    dW1 = 1.0 / m * np.dot(dZ1, X.T)
    db1 = 1.0 / m * np.sum(dZ1, axis=1, keepdims=True)

    grads = {"dW1": dW1,
             "db1": db1,
             "dW2": dW2,
             "db2": db2}

    return grads


def update_parameters(parameters, grads, learning_rate=1.2):
    """
    Updates parameters using the gradient descent update rule given above

    Arguments:
    parameters -- python dictionary containing your parameters
    grads -- python dictionary containing your gradients

    Returns:
    parameters -- python dictionary containing your updated parameters
    """
    # Retrieve each parameter from the dictionary "parameters"
    W1 = parameters["W1"]
    b1 = parameters["b1"]
    W2 = parameters["W2"]
    b2 = parameters["b2"]

    # Retrieve each gradient from the dictionary "grads"
    dW1 = grads["dW1"]
    db1 = grads["db1"]
    dW2 = grads["dW2"]
    db2 = grads["db2"]

    # Update rule for each parameter
    W1 -= learning_rate * dW1
    b1 -= learning_rate * db1
    W2 -= learning_rate * dW2
    b2 -= learning_rate * db2

    parameters = {"W1": W1,
                  "b1": b1,
                  "W2": W2,
                  "b2": b2}

    return parameters


def nn_model(X, Y, n_h, num_iterations=10000, print_cost=False):
    """
    Arguments:
    X -- dataset of shape (2, number of examples)
    Y -- labels of shape (1, number of examples)
    n_h -- size of the hidden layer
    num_iterations -- Number of iterations in gradient descent loop
    print_cost -- if True, print the cost every 1000 iterations

    Returns:
    parameters -- parameters learnt by the model. They can then be used to predict.
    """

    np.random.seed(3)
    n_x = layer_sizes(X, Y)[0]
    n_y = layer_sizes(X, Y)[2]

    # Initialize parameters, then retrieve W1, b1, W2, b2.
    # Inputs: "n_x, n_h, n_y". Outputs = "W1, b1, W2, b2, parameters".
    parameters = initialize_parameters(n_x, n_h, n_y)
    W1 = parameters["W1"]
    b1 = parameters["b1"]
    W2 = parameters["W2"]
    b2 = parameters["b2"]

    # Loop (gradient descent)

    for i in range(0, num_iterations):

        # Forward propagation. Inputs: "X, parameters". Outputs: "A2, cache".
        A2, cache = forward_propagation(X, parameters)

        # Cost function. Inputs: "A2, Y, parameters". Outputs: "cost".
        cost = compute_cost(A2, Y, parameters)

        # Backpropagation. Inputs: "parameters, cache, X, Y". Outputs: "grads".
        grads = backward_propagation(parameters, cache, X, Y)

        # Gradient descent parameter update. Inputs: "parameters, grads". Outputs: "parameters".
        parameters = update_parameters(parameters, grads)

        # Print the cost every 1000 iterations
        if print_cost and i % 1000 == 0:
            print("Cost after iteration %i: %f" % (i, cost))

    return parameters


def predict(parameters, X):
    """
    Using the learned parameters, predicts a class for each example in X

    Arguments:
    parameters -- python dictionary containing your parameters
    X -- input data of size (n_x, m)

    Returns
    predictions -- vector of predictions of our model (red: 0 / blue: 1)
    """

    # Computes probabilities using forward propagation,
    # and classifies to 0/1 using 0.5 as the threshold.
    A2, cache = forward_propagation(X, parameters)
    predictions = np.array([1 if (i > 0.5) else 0 for i in A2[0]])

    return predictions




# -*- coding: utf-8 -*-
"""
——————————————————————————————————————————————————————————————————————————————————————
Logistic Regression
——————————————————————————————————————————————————————————————————————————————————————
"""

import numpy as np
import matplotlib.pyplot as plt

from zb.functions import sigmoid


# step 1. initialize parameters with zeros

def initialize_with_zeros(dim):
    """
    This function creates a vector of zeros of shape (dim, 1) for w and initializes b to 0.

    Argument:
    dim     size of the w vector we want (or number of parameters in this case)

    Returns:
    w       initialized vector of shape (dim, 1)
    b       initialized scalar (corresponds to the bias)
    """
    w = np.zeros((dim, 1))
    b = 0
    return w, b


# step 2. compute the cost function and its gradient

def propagate(w, b, X, Y):
    """
    Implement the cost function and its gradient for the propagation

    Arguments:
    w       weights, a numpy array of size (dim, 1)
    b       bias, a scalar
    X       data of size (dim, number of examples)
    Y       true "label" vector of size (1, number of examples)

    Return:
    cost    negative log-likelihood cost for logistic regression
    dw      gradient of the loss with respect to w, thus same shape as w
    db      gradient of the loss with respect to b, thus same shape as b

    """
    m = X.shape[1]
    A = sigmoid(np.dot(w.T, X) + b)
    cost = -1 / m * np.sum(Y * np.log(A) + (1 - Y) * np.log(1 - A))

    # BACKWARD PROPAGATION (TO FIND GRAD)
    dw = 1.0 / m * np.dot(X, (A - Y).T)
    db = 1.0 / m * np.sum(A - Y)

    assert (dw.shape == w.shape)
    assert (db.dtype == float)
    cost = np.squeeze(cost)
    assert (cost.shape == ())

    grads = {"dw": dw,
             "db": db}

    return grads, cost


# step 3. update parameters with gradient descending (train LR model)

def LR_train(w, b, X, Y, num_iterations, learning_rate, print_cost=True):
    """
    This function optimizes w and b by running a gradient descent algorithm

    Arguments:
    w       weights, a numpy array of size (dim, 1)
    b       bias, a scalar
    X       data of shape (dim, number of examples)
    Y       true "label" vector, of shape (1, number of examples)
    num_iterations      number of iterations of the optimization loop
    learning_rate       learning rate of the gradient descent update rule
    print_cost          True to print the loss every 100 steps

    Returns:
    params      dictionary containing the weights w and bias b
    grads       dictionary containing the gradients of the weights and bias with respect to the cost function
    costs       list of all the costs computed during the optimization, this will be used to plot the learning curve.
    """

    costs = []

    for i in range(num_iterations):

        # Cost and gradient calculation
        grads, cost = propagate(w, b, X, Y)

        # Retrieve derivatives from grads
        dw = grads["dw"]
        db = grads["db"]

        # update rule
        w = w - learning_rate * dw
        b = b - learning_rate * db

        # Record the costs
        if i % 100 == 0:
            costs.append(cost)

        # Print the cost every 100 training examples
        if print_cost and i % 100 == 0:
            print("Cost after iteration %i: %f" % (i, cost))

    params = {"w": w,
              "b": b}

    grads = {"dw": dw,
             "db": db}

    return params, grads, costs


# step 4. use trained model to predict

def LR_predict(w, b, X):
    """
    Predict whether the label is 0 or 1 using learned logistic regression parameters (w, b)

    Arguments:
    w -- weights, a numpy array of size (num_px * num_px * 3, 1)
    b -- bias, a scalar
    X -- data of size (num_px * num_px * 3, number of examples)

    Returns:
    Y_prediction -- a numpy array (vector) containing all predictions (0/1) for the examples in X
    """

    m = X.shape[1]
    Y_prediction = np.zeros((1, m))
    w = w.reshape(X.shape[0], 1)

    A = sigmoid(np.dot(w.T, X) + b)

    for i in range(A.shape[1]):
        if A[0, i] > 0.5:
            Y_prediction[0, i] = 1.0
        else:
            Y_prediction[0, i] = 0.0

    assert (Y_prediction.shape == (1, m))

    return Y_prediction


# final: merge all into a model

def model(X_train, Y_train, X_test, Y_test, num_iterations=2000,
          learning_rate=0.5, print_cost=True):
    """
    Builds the logistic regression model by calling the function
    implemented above

    Arguments:
    X_train     training set represented by a numpy array of shape (dim, m_train)
    Y_train     training labels represented by a numpy array (vector) of shape (1, m_train)
    X_test      test set represented by a numpy array of shape (dim, m_test)
    Y_test      test labels represented by a numpy array (vector) of shape (1, m_test)
    num_iterations  hyperparameter representing the number of iterations to optimize the parameters
    learning_rate   hyperparameter representing the learning rate used in the update rule of optimize()
    print_cost      Set to true to print the cost every 100 iterations

    Returns:
    d -- dictionary containing information about the model.
    """
    print('X_train shape:', X_train.shape)

    # initialize parameters with zeros
    w, b = initialize_with_zeros(X_train.shape[0])

    print('w shape:', w.shape)
    # Gradient descent
    parameters, grads, costs = LR_train(w, b, X_train, Y_train, num_iterations,
                                        learning_rate, print_cost)

    # Retrieve parameters w and b from dictionary "parameters"
    w = parameters["w"]
    b = parameters["b"]

    print('w shape params:', w.shape)

    Y_prediction_test = LR_predict(w, b, X_test)
    Y_prediction_train = LR_predict(w, b, X_train)

    # Print train/test Errors
    print("train accuracy: {} %".format(100 - np.mean(np.abs(Y_prediction_train - Y_train)) * 100))
    print("test accuracy: {} %".format(100 - np.mean(np.abs(Y_prediction_test - Y_test)) * 100))

    d = {"costs": costs,
         "Y_prediction_test": Y_prediction_test,
         "Y_prediction_train": Y_prediction_train,
         "w": w,
         "b": b,
         "learning_rate": learning_rate,
         "num_iterations": num_iterations}

    return d


def plot_lc(d):
    """plot learning curve

    Arguments:
    d       Returned by model function. Dictionary containing information about the model.
    """
    costs = np.squeeze(d['costs'])
    plt.plot(costs)
    plt.ylabel('cost')
    plt.xlabel('iterations (per hundreds)')
    plt.title("Learning rate =" + str(d["learning_rate"]))
    plt.show()



# -*- coding: utf-8 -*-

import pandas as pd
import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np

mpl.rc('axes', facecolor='#6E838A')
mpl.rc('axes', edgecolor='#737373')

# datas in
datas = pd.read_csv('*.csv', encoding='gbk')


class stacked_bar_plot:
    def __init__(self, datas):
        self.version = '1.0'
        self.datas = datas

    def parse_datas(datas=datas):
        d_values = datas.iloc[:, 2].values.reshape((24, 3))
        x_labels = datas.iloc[:, 0].unique()
        legends = datas.iloc[:, 1].unique()
        return d_values, x_labels, legends

    d_values, x_labels, legends = parse_datas(datas)

    # 重整数据
    def compute_ratio(x):
        """
        计算每一类数据的占比
        """
        sum_ = sum(x)
        ratios = []
        for i in x:
            ratio = i / sum_
            ratios.append(ratio)
        return ratios

    per_ratio = np.apply_along_axis(compute_ratio, 1, d_values)

    def stacked_bar(self, width=0.6):
        per_ratio = self.per_ratio * 50
        # split into three 
        gcq_ratio = per_ratio[:, 0]
        lwq_ratio = per_ratio[:, 2]
        other_ratio = per_ratio[:, 1]
        # set bar left edge
        ind = np.arange(24)

        plt.figure(figsize=(20, 7), facecolor='#6E838A')
        plt.ylim(0, 55)
        plt.bar(ind + 0.5, gcq_ratio, width, color='#588fc1')
        plt.bar(ind + 0.5, lwq_ratio, width, bottom=gcq_ratio, color='#6a9f46')
        plt.bar(ind + 0.5, other_ratio, width, bottom=gcq_ratio + lwq_ratio, color='#e7af08')

        # add x,y ticks
        xy_font = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\STKAITI.TTF', size=18)
        x_labels = datas.utility.unique()
        plt.xticks(ind + 0.8, x_labels, fontproperties=xy_font, rotation=90)
        plt.yticks(np.arange(0, 50, 5), np.arange(0, 100, 10), fontproperties=xy_font)

        # add legend
        l_font = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\STKAITI.TTF', size=16)
        plt.legend(('*', '*', '*'), prop=l_font,
                   ncol=3, frameon=False, loc='upper center')

        for label in plt.axes().get_xticklabels() + plt.axes().get_yticklabels():
            label.set_color('white')

        # add title
        t_font = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\STKAITI.TTF', size=28)
        plt.title('*', fontproperties=t_font, color='white')

        # add x label
        plt.ylabel('*', fontproperties=xy_font, color='white')
        plt.show()



# -*- coding: utf-8 -*-

from urllib.request import urlopen, Request
from pdfminer.pdfparser import PDFParser, PDFDocument
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import *
from pdfminer.pdfinterp import PDFTextExtractionNotAllowed


def pdf2text(path, password="", headers=None):
    """从pdf文件中提取文本

    :param path: str
        pdf文件在本地的地址或者url地址
    :param password: str
        pdf文件的打开密码，默认为空
    :param headers: dict
        请求url所需要的 header，默认值为 None
    :return: text
    """
    if path.startswith("http"):
        if headers:
            request = Request(url=path, headers=headers)
        else:
            request = Request(url=path, headers={})
        fp = urlopen(request)  # 打开在线PDF文档
    else:
        fp = open(path, 'rb')

    parser = PDFParser(fp)
    doc = PDFDocument()
    parser.set_document(doc)
    doc.set_parser(parser)

    # 提供初始化密码
    # 如果没有密码 就创建一个空的字符串
    doc.initialize(password)

    # 检测文档是否提供txt转换，不提供就忽略
    if not doc.is_extractable:
        raise PDFTextExtractionNotAllowed
    else:
        rsrcmgr = PDFResourceManager()
        laparams = LAParams()
        device = PDFPageAggregator(rsrcmgr, laparams=laparams)
        interpreter = PDFPageInterpreter(rsrcmgr, device)

        text = []
        for page in doc.get_pages():
            interpreter.process_page(page)
            layout = device.get_result()
            for x in layout:
                if isinstance(x, LTTextBoxHorizontal):
                    text.append(x.get_text())
    return text





# -*- coding: utf-8 -*-
import os
import shutil


def clean_folder(folder):
    """清空文件夹，并确保文件夹存在"""
    if os.path.exists(folder):
        shutil.rmtree(folder)
    os.mkdir(folder)


def insure_folder_exists(folder):
    """确保文件夹存在"""
    if not os.path.exists(folder):
        os.mkdir(folder)


# def get_file_list(folder_path, form='view', graph=True):
def get_file_list(folder_path):
    """将folder_path文件夹下的所有文件的相对路径保存到txt文件中

    params
    ---------
    folder_path     文件夹路径
    form            文件名保存形式，可选 'view' 'full'
    """
    import os
    f = open(folder_path + '\\file_list.txt', 'w')
    # f.write('根目录：\n')
    # 写入文件相对路径
    for root, dirs, files in os.walk(folder_path):
        x = ''
        for file in files:
            if root != x:  # 如果不是同一个root，换行
                f.write('\n' + root.replace(folder_path + '\\', '') + '\n')
            file_1 = os.path.join(root, file)
            file_2 = file_1.replace(root + '\\', '')
            f.write('  |-- ' + file_2 + '\n')
            x = root
    f.close()



# -*- coding: utf-8 -*-
"""
——————————————————————————————————————————————————————————————————————————————————————————
Clustering by Fast Search and Find of Density Peaks

输入数据：
    pd.DataFrame 每一行表示一个点

——————————————————————————————————————————————————————————————————————————————————————————
"""


# step 1. 计算距离矩阵

def cal_distance(xi, xj):
    pass


# step 2. 计算每一个点的局部密度
def cal_loc_density(ddi, dc):
    pass


# step 3. 计算每一个点与其他具有更高局部密度的点集的最小距离
def cal_min_distance(ddi, ld):
    pass


# step 4. 做决策图，确定类簇的数量
def decision_cluster_num():
    pass

# step 5. 归类



# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt


def plot_decision_boundary(model, X, y):
    # Set min and max values and give it some padding
    x_min, x_max = X[0, :].min() - 1, X[0, :].max() + 1
    y_min, y_max = X[1, :].min() - 1, X[1, :].max() + 1
    h = 0.01
    # Generate a grid of points with distance h between them
    xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))
    # Predict the function value for the whole grid
    Z = model(np.c_[xx.ravel(), yy.ravel()])
    Z = Z.reshape(xx.shape)
    # Plot the contour and training examples
    plt.contourf(xx, yy, Z, cmap=plt.cm.Spectral)
    plt.ylabel('x2')
    plt.xlabel('x1')
    plt.scatter(X[0, :], X[1, :], c=y, cmap=plt.cm.Spectral)


# -*- coding: utf-8 -*-
"""
zb.nlp.kwe - 关键词提取

kwe = key word extract
====================================================================
"""


import jieba.analyse
from textrank4zh import TextRank4Keyword


def kwe_by_tfidf(text, top=10, pos=None, with_weight=False):
    """

    :param text:
    :param top:
    :param pos:
    :param with_weight:
    :return:
    """
    if pos is None:
        pos = ['ns', 'n', 'vn', 'v', 'nr']
    kw = jieba.analyse.extract_tags(text, topK=top, withWeight=True,
                                    allowPOS=pos)
    if not with_weight:
        kw = [x[0] for x in kw]
    return kw


def kwe_by_textrank(text, top=10, pos=None, with_weight=False, key_phrase=False):
    """

    :param text:
    :param top:
    :param pos:
    :param with_weight:
    :param key_phrase:
    :return:
    """
    if pos is None:
        pos = ['ns', 'n', 'vn', 'v', 'nr']
    tr4k = TextRank4Keyword(allow_speech_tags=pos)
    tr4k.analyze(text)
    kw = tr4k.get_keywords(num=top, word_min_len=2)
    if not with_weight:
        kw = [x['word'] for x in kw]
    else:
        kw = [(x['word'], x['weight']) for x in kw]

    if key_phrase:
        kp = tr4k.get_keyphrases(keywords_num=top, min_occur_num=2)
        return kw, kp
    else:
        return kw



# -*- coding: utf-8 -*-

import numpy as np


def sigmoid(z):
    """
    Compute the sigmoid of z

    Arguments:
    z       A scalar or numpy array of any size.

    Return:
    s       sigmoid(z)
    """
    s = 1.0 / (1.0 + np.exp(-z))
    return s


def sigmoid_derivative(x):
    """
    Compute the derivative of the sigmoid function with respect to its input x.

    Arguments:
    x -- A scalar or numpy array

    Return:
    ds -- Your computed gradient.
    """
    s = sigmoid(x)
    ds = s * (1.0 - s)
    return ds

